/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.examples.mailreaderjpa;

import java.util.List;
import java.util.Map;
import org.apache.mailreaderjpa.Protocol;
import org.apache.mailreaderjpa.Subscription;
import org.apache.mailreaderjpa.User;
import org.apache.shale.view.AbstractViewController;

/**
 * <p>Backing bean for the <code>/detail.jsp</code> view.</p>
 */
public class Detail extends AbstractViewController {
    

    // ------------------------------------------------------ Manifest Constants


    /**
     * <p>Cache key under which we save the current primary key.</p>
     */
    private static final String ID_KEY = "id";


    /**
     * <p>Cache key under which we save the current transaction mode.</p>
     */
    private static final String MODE_KEY = "mode";


    /**
     * <p>Cache key under which we save the <code>Subscription</code> instance
     * we are creating or editing.</p>
     */
    private static final String SUBSCRIPTION_KEY = "user";


    // ------------------------------------------------------- Public Properties


    /**
     * <p>The {@link Domains} for this application.  This value
     * will be injected, based on the managed bean configuration.</p>
     */
    private Domains domains = null;


    /**
     * <p>Return the {@link Domains} for this application.</p>
     */
    public Domains getDomains() {
        return this.domains;
    }


    /**
     * <p>Set the {@link Domains} for this application.</p>
     *
     * @param domains The new domains instance
     */
    public void setDomains(Domains domains) {
        this.domains = domains;
    }


    /**
     * <p>The primary key of the row to be processed.</p>
     */
    private int id = 0;


    /**
     * <p>Return the primary key of the row to be processed.</p>
     */
    private int getId() {
        return this.id;
    }


    /**
     * <p>Set the primary key of the row to be processed.</p>
     *
     * @param id The new primary key
     */
    public void setId(int id) {
        this.id = id;
    }


    /**
     * <p>The business {@link Logic} for this application.  This value
     * will be injected, based on the managed bean configuration.</p>
     */
    private Logic logic = null;


    /**
     * <p>Return the business {@link Logic} for this application.</p>
     */
    public Logic getLogic() {
        return this.logic;
    }


    /**
     * <p>Set the business {@link Logic} for this application.</p>
     *
     * @param logic The new business logic instance
     */
    public void setLogic(Logic logic) {
        this.logic = logic;
    }


    /**
     * <p>The transaction mode for the current request (CREATE or EDIT
     * or DELETE or DISPLAY).  Default mode is DISPLAY.</p>
     */
    private String mode = Constants.DISPLAY_MODE;


    /**
     * <p>Return the transaction mode (CREATE or EDIT) for this request.</p>
     */
    public String getMode() {
        return this.mode;
    }


    /**
     * <p>Set the transaction mode (CREATE or EDIT) for this request.</p>
     *
     * @param mode The new transaction mode
     */
    public void setMode(String mode) {
        this.mode = mode;
    }


    /**
     * <p>Return <code>true</code> if the input fields should be rendered
     * as read only.</p>
     */
    public boolean isReadOnly() {
        return Constants.DELETE_MODE.equals(getMode())
            || Constants.DISPLAY_MODE.equals(getMode());
    }


    /**
     * <p>The per-user {@link State} instance we are associated with.  This
     * value will be injected, based on the managed bean configuration.</p>
     */
    private State state = null;


    /**
     * <p>Return the per-user {@link State} instance we are associated with.</p>
     */
    public State getState() {
        return this.state;
    }


    /**
     * <p>Set the per-user {@link State} instance we are associated with.</p>
     *
     * @param state The new State instance
     */
    public void setState(State state) {
        this.state = state;
    }


    /**
     * <p>The <code>Subscription</code> instance being created or edited.</p>
     */
    private Subscription subscription = null;


    /**
     * <p>Return the <code>Subscription</code> instance we are creating or editing.</p>
     */
    public Subscription getSubscription() {
        return this.subscription;
    }


    // -------------------------------------------------------- Lifecycle Events


    /**
     * <p>Initialize primary key and transaction mode from the request URI,
     * if specified.</p>
     */
    public void init() {

        log("Detail.init(" + isPostBack() + ")");

        String id = getRequestParameter("id");
        if (id != null) {
            try {
                setId(Integer.valueOf(id));
            } catch (NumberFormatException e) {
                setId(0);
            }
        }

        String mode = getRequestParameter("mode");
        if ((mode != null) && (mode.length() > 0)) {
            setMode(mode);
        }

    }


    /**
     * <p>Restore the <code>User</code> instance we are creating or editing,
     * creating a new one if necessary.</p>
     */
    public void preprocess() {

        log("Detail.preprocess(" + isPostBack() + ")");

        String mode = (String) retrieveData(MODE_KEY);
        if (mode != null) {
            setMode(mode);
        }

        Subscription subscription = (Subscription) retrieveData(SUBSCRIPTION_KEY);
        if (subscription != null) {
            this.subscription = subscription;
        } else {
            setup();
        }

    }


    /**
     * <p>Cache the <code>Subscription</code> instance we are creating or editing,
     * as well as the transaction model</p>
     */
    public void prerender() {

        log("Detail.prerender(" + isPostBack() + ")");

        // Create a new user instance if necessary (i.e. we navigated
        // to this page directly, or we are in CREATE mode)
        if (getSubscription() == null) {
            setup();
        }

        // Cache the current transaction mode and user instance
        saveData(MODE_KEY, getMode());
        saveData(SUBSCRIPTION_KEY, getSubscription());

    }


    /**
     * <p>Release resources used during this request.</p>
     */
    public void destroy() {

        log("Detail.destroy(" + isPostBack() + ")");

        ; // FIXME - destroy()

    }


    // ------------------------------------------------------------- View Events


    /**
     * <p>Cancel saving the updated subscription information.</p>
     */
    public String cancel() {

        return "registration";

    }


    /**
     * <p>Save away the updated subscription information.</p>
     */
    public String save() {

        try {

            if (Constants.CREATE_MODE.equals(getMode())) {
                getLogic().createSubscription(getSubscription());
            } else if (Constants.DELETE_MODE.equals(getMode())) {
                getLogic().deleteSubscription(getSubscription());
            } else if (Constants.EDIT_MODE.equals(getMode())) {
                this.subscription = getLogic().updateSubscription(getSubscription());
            }

        } catch (Exception e) {

            error("Exception saving subscription information: " + e.getMessage());
            log("Exception saving subscription information", e);
            return null;
        }

        return "registration";

    }


    // --------------------------------------------------------- Support Methods


    /**
     * <p>Set up the <code>Subscription</code> instance we are creating and editing,
     * by re-finding via the specified primary key (if any), or by creating
     * a new instance.</p>
     */
    private void setup() {

        if (getId() != 0) {
            this.subscription = getLogic().findSubscriptionById(getId());
        } else {
            this.subscription = new Subscription();
            this.subscription.setUser(getState().getUser());
            this.subscription.setProtocol((Protocol) getDomains().getProtocols()[0].getValue());
        }

    }


}
