/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.examples.mailreaderjpa;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceException;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import org.apache.mailreaderjpa.Protocol;
import org.apache.mailreaderjpa.Subscription;
import org.apache.mailreaderjpa.User;
import org.apache.shale.view.AbstractApplicationBean;

/**
 * <p>Business logic for this example application.  This bean is designed
 * to be a managed bean with scope "application", so the implementation of
 * each public method must be done in a threadsafe manner.</p>
 *
 * <p><strong>USAGE NOTE</strong>:  The public methods of this class may throw
 * unchecked exceptions if the underlying persistence framework has problems.
 * If you intend to create graceful UIs for such exceptions, be sure to
 * enclose calls to these methods in try/catch blocks.</p>
 */
public class Logic extends AbstractApplicationBean {
    

    // ------------------------------------------------------ Manifest Constants


    /**
     * <p>Name of the persistence context we will be using.</p>
     */
    private static final String PERSISTENCE_CONTEXT_NAME = "MailReaderJpa";


    /**
     * <p>Named query for selecting all <code>Protocol</code> instances.</p>
     */
    private static final String PROTOCOL_QUERY = "Protocol.findAll";


    /**
     * <p>Named query for finding a <code>User</code> by username.</p>
     */
    private static final String USERNAME_QUERY = "User.findByUsername";


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The entity manager factory we will be using.  This field is package scoped
     * for purposes of unit testing.</p>
     */
    @PersistenceUnit(name=PERSISTENCE_CONTEXT_NAME)
    EntityManagerFactory emf;


    // -------------------------------------------------------- Lifecycle Events


    /**
     * <p>Log initialization of this business logic bean.</p>
     */
    public void init() {

        log("Initializing business logic bean");

    }


    /**
     * <p>On application shutdown, close the entity manager factory.</p>
     */
    public void destroy() {

        log("Finalizing business logic bean");
        emf.close();

    }


    // ------------------------------------------------------- Public Properties


    /**
     * <p>Return a <code>List</code> of the valid <code>Protocol</code>s for
     * this application.  If no valid protocols are defined, a zero length
     * list will be returned.</p>
     */
    public List<Protocol> getProtocols() {

        EntityManager em = entityManager();
        try {
            List<Protocol> protocols =
              em.createNamedQuery(PROTOCOL_QUERY).getResultList();
            if (protocols == null) {
                protocols = new ArrayList<Protocol>();
            }
            return protocols;
        } finally {
            em.close();
        }

    }


    // -------------------------------------------------- Business Logic Methods


    /**
     * <p>Authenticate the specified logon credentials, returning a corresponding
     * <code>User</code> if they are valid or <code>null</code> otherwise.</p>
     *
     * @param username Username to authenticate
     * @param password Password to authenticate
     */
    public User authenticate(String username, String password) {

        User user = findUserByUsername(username);
        if ((user != null)
            && user.getPassword().equals(password)) {
            return user;
        } else {
            return null;
        }

    }


    /**
     * <p>Bootstrap the contents of the database, if necessary.  This is
     * strictly a convenience mechanism so that the out-of-the-box content
     * of the data matches that in the original demo.  This is not typical
     * of what you would need in a normal application.</p>
     */
    public void bootstrap() {

        EntityManager em = entityManager();
        EntityTransaction et = null;

        try {
        
            // Determine whether bootstrapping is required
            List<Protocol> protocols = em.createNamedQuery(PROTOCOL_QUERY).getResultList();
            if ((protocols != null) && (protocols.size() > 0)) {
                return;
            }

            // Start a transaction since we will be updating the database
            et = em.getTransaction();
            et.begin();

            // Create the basic protocol data
            Protocol protocol1 = null;
            protocol1 = new Protocol();
            protocol1.setDescription("IMAP Protocol");
            em.persist(protocol1);
            Protocol protocol2 = new Protocol();
            protocol2.setDescription("POP3 Protocol");
            em.persist(protocol2);

            // Set up the initial user and subscriptions
            User user = new User();
            user.setUsername("user");
            user.setPassword("pass");
            user.setFullName("John Q. User");
            user.setFromAddress("John.User@somewhere.com");
            List<Subscription> list = new ArrayList<Subscription>();
            user.setSubscriptions(list);
            Subscription sub = null;
            sub = new Subscription();
            sub.setUser(user);
            sub.setHost("mail.yahoo.com");
            sub.setProtocol(protocol1);
            sub.setUsername("jquser");
            sub.setPassword("foo");
            list.add(sub);
            sub = new Subscription();
            sub.setUser(user);
            sub.setHost("mail.hotmail.com");
            sub.setProtocol(protocol2);
            sub.setUsername("user1234");
            sub.setPassword("bar");
            list.add(sub);
            em.persist(user);

            // Commit the database updates
            et.commit();

        } finally {

            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();

        }

    }


    /**
     * <p>Add the specified <code>Subscription</code> to the persistent database.</p>
     *
     * @param subscription Subscription to be added
     */
    public void createSubscription(Subscription subscription) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            User user = em.find(User.class, subscription.getUser().getId());
            user.addSubscription(subscription);
            em.persist(subscription);
            et.commit();
        } catch (Exception e) {
            log("Exception in createSubscription()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    /**
     * <p>Add the specified <code>User</code> (and any associated child
     * <code>Subscription</code>s) to the persistent database.</p>
     *
     * @param user User to be added
     */
    public void createUser(User user) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            em.persist(user);
            et.commit();
        } catch (Exception e) {
            log("Exception in createUser()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    /**
     * <p>Delete the specified <code>Subscription</code> from the persistent database.</p>
     *
     * @param subscription Subscription to be deleted
     */
    public void deleteSubscription(Subscription subscription) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            User user = em.find(User.class, subscription.getUser().getId());
            user.removeSubscription(subscription);
            subscription = em.merge(subscription);
            em.remove(subscription);
            et.commit();
        } catch (Exception e) {
            log("Exception in deleteSubscription()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    /**
     * <p>Delete the specified <code>User</code> (and any associated child
     * <code>Subscription</code>s) from the persistent database.</p>
     *
     * @param user User to be deleted
     */
    public void deleteUser(User user) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            user = em.merge(user);
            em.remove(user);
            et.commit();
        } catch (Exception e) {
            log("Exception in deleteUser()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    /**
     * <p>Return the <code>Protocol</code> with the specified id, if any;
     * otherwise, return <code>null</code>.</p>
     *
     * @param id Protocol id to look up
     */
    public Protocol findProtocolById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(Protocol.class, id);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Return the <code>Subscription</code> with the specified id, if any;
     * otherwise, return <code>null</code>.</p>
     *
     * @param id Subscription id to look up
     */
    public Subscription findSubscriptionById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(Subscription.class, id);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Return the <code>User</code> with the specified id, if any;
     * otherwise, return <code>null</code>.</p>
     *
     * @param id User id to look up
     */
    public User findUserById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(User.class, id);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Return the <code>User</code> with the specified username, if any;
     * otherwise, return <code>null</code>.</p>
     *
     * @param username Username to look up
     */
    public User findUserByUsername(String username) {

        EntityManager em = entityManager();
        User user = null;
        try {
            Query query = em.createNamedQuery(USERNAME_QUERY);
            query.setParameter("username", username);
            user = (User) query.getSingleResult();
            return user;
        } catch (NoResultException e) {
            return null;
        } finally {
            em.close();
        }

    }


    /**
     * <p>Update the specified <code>Subscription</code> in the persistent database.</p>
     *
     * @param subscription Subscription to be updated
     *
     * @return The updated instance
     */
    public Subscription updateSubscription(Subscription subscription) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            subscription = em.merge(subscription);
            et.commit();
            return subscription;
        } catch (Exception e) {
            log("Exception in updateSubscription()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    /**
     * <p>Update the specified <code>User</code> (and any associated child
     * <code>Subscription</code>s) in the persistent database.</p>
     *
     * @param user User to be updated
     *
     * @return The updated instance
     */
    public User updateUser(User user) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            user = em.merge(user);
            et.commit();
            return user;
        } catch (Exception e) {
            log("Exception in updateUser()", e);
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }


    }


    // --------------------------------------------------------- Support Methods


    /**
     * <p>Return an <code>EntityManager</code> that can be used to perform
     * persistence operations.</p>
     */
    private EntityManager entityManager() {

        return emf.createEntityManager();

    }


}
